@@ -1,57 +1,28 @@
-# coding: utf-8
-# /*##########################################################################
-# Copyright (C) 2016-2022 European Synchrotron Radiation Facility
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
-#
-#############################################################################
-
-"""contains EDFTomoScan, class to be used with EDF acquisition"""
-
-
-__authors__ = ["H.Payno"]
-__license__ = "MIT"
-__date__ = "10/10/2019"
-
+"""Scan dedicated for bliss format - based on EDF files"""
 
+import copy
+import io
+import json
+import logging
 import os
 import re
+from typing import Iterable, Optional, Union
+
 import fabio
-import copy
-from lxml import etree
-import json
-import io
-from typing import Union, Iterable, Optional
-
 import numpy
-
+from lxml import etree
 from silx.io.url import DataUrl
-from silx.utils.deprecation import deprecated
+
+from pyunitsystem.metricsystem import MetricSystem
+
+from tomoscan.utils.io import deprecated, deprecated_warning
 from tomoscan.esrf.identifier.edfidentifier import EDFTomoScanIdentifier
-from tomoscan.scanbase import TomoScanBase
-from tomoscan.scanbase import Source
-from .utils import get_parameters_frm_par_or_info, extract_urls_from_edf
-from tomoscan.unitsystem.metricsystem import MetricSystem
-from tomoscan.utils import docstring
-from .framereducer import EDFFrameReducer
-import logging
 from tomoscan.identifier import ScanIdentifier
+from tomoscan.scanbase import Source, TomoScanBase
+from tomoscan.utils import docstring
+
+from .framereducer import EDFFrameReducer
+from .utils import extract_urls_from_edf, get_parameters_frm_par_or_info
 
 _logger = logging.getLogger(__name__)
 
@@ -332,17 +303,20 @@ class EDFTomoScan(TomoScanBase):
     @property
     @docstring(TomoScanBase.x_translation)
     def x_translation(self) -> Union[None, tuple]:
-        raise NotImplementedError("Not supported for EDF")
+        _logger.warning("x_translation Not supported for EDF")
+        return None
 
     @property
     @docstring(TomoScanBase.y_translation)
     def y_translation(self) -> Union[None, tuple]:
-        raise NotImplementedError("Not supported for EDF")
+        _logger.warning("y_translation Not supported for EDF")
+        return None
 
     @property
     @docstring(TomoScanBase.z_translation)
     def z_translation(self) -> Union[None, tuple]:
-        raise NotImplementedError("Not supported for EDF")
+        _logger.warning("z_translation Not supported for EDF")
+        return None
 
     @property
     @docstring(TomoScanBase.ff_interval)
@@ -716,6 +690,33 @@ class EDFTomoScan(TomoScanBase):
         """
         return None
 
+    @property
+    @docstring(TomoScanBase.title)
+    def title(self) -> Union[None, str]:
+        """
+
+        :return: title
+        """
+        return None
+
+    @property
+    @docstring(TomoScanBase.source_name)
+    def source_name(self) -> Union[None, str]:
+        """
+
+        :return: source name
+        """
+        return None
+
+    @property
+    @docstring(TomoScanBase.source_type)
+    def source_type(self) -> Union[None, str]:
+        """
+
+        :return: source type
+        """
+        return None
+
     @property
     @docstring(TomoScanBase.distance)
     def distance(self) -> Union[None, float]:
@@ -912,7 +913,7 @@ class EDFTomoScan(TomoScanBase):
             new_key = key
             index = 2
             while new_key in keys:
-                new_key = "%s (%s)" % (key, index)
+                new_key = f"{key} ({index})"
                 index += 1
             return new_key
 
@@ -949,10 +950,29 @@ class EDFTomoScan(TomoScanBase):
 
     @property
     def x_flipped(self) -> bool:
+        deprecated_warning(
+            type_="property",
+            name="x_flipped",
+            replacement="get_detector_transformations",
+            since_version="1.3",
+        )
         return None
 
     @property
     def y_flipped(self) -> bool:
+        deprecated_warning(
+            type_="property",
+            name="y_flipped",
+            replacement="detector_transformations",
+            since_version="1.3",
+        )
+        return None
+
+    @property
+    def detector_transformations(self) -> Optional[tuple]:
+        """
+        not handled for EDF
+        """
         return None
 
     def _reload_projections(self):
@@ -1188,6 +1208,7 @@ class EDFTomoScan(TomoScanBase):
         output_urls: tuple = REDUCED_DARKS_DATAURLS,
         darks_infos=None,
         metadata_output_urls=REDUCED_DARKS_METADATAURLS,
+        overwrite: bool = False,
     ):
         if len(darks) > 1:
             _logger.warning(
@@ -1198,6 +1219,7 @@ class EDFTomoScan(TomoScanBase):
             output_urls=output_urls,
             darks_infos=darks_infos,
             metadata_output_urls=metadata_output_urls,
+            overwrite=overwrite,
         )
 
     @docstring(TomoScanBase)
@@ -1214,6 +1236,10 @@ class EDFTomoScan(TomoScanBase):
             return_as_url=return_as_url,
             return_info=return_info,
         )
+        if return_info is True:
+            darks, info = darks
+        else:
+            info = None
         # for edf we don't expect dark to have a index and we set it by default at frame index 0
         if None in darks:
             dark_frame = darks[None]
@@ -1222,7 +1248,10 @@ class EDFTomoScan(TomoScanBase):
                 _logger.warning("Two frame found for index 0")
             else:
                 darks[0] = dark_frame
-        return darks
+        if return_info is True:
+            return darks, info
+        else:
+            return darks
 
     @docstring(TomoScanBase)
     def save_reduced_flats(
@@ -1231,12 +1260,14 @@ class EDFTomoScan(TomoScanBase):
         output_urls: tuple = REDUCED_FLATS_DATAURLS,
         flats_infos=None,
         metadata_output_urls=REDUCED_FLATS_METADATAURLS,
+        overwrite: bool = False,
     ) -> dict:
         super().save_reduced_flats(
             flats=flats,
             output_urls=output_urls,
             flats_infos=flats_infos,
             metadata_output_urls=metadata_output_urls,
+            overwrite=overwrite,
         )
 
     @docstring(TomoScanBase)
