@@ -28,20 +28,20 @@ __license__ = "MIT"
 __date__ = "04/01/2022"
 
 
-from typing import Optional
-from tomoscan.framereducerbase import (
-    REDUCER_TARGET,
-    FrameReducerBase,
-    ReduceMethod,
-)
-from tomoscan.scanbase import TomoScanBase, ReducedFramesInfos
-from lxml import etree
-import os
-import numpy
-import fabio
 import logging
+import os
 import re
 from glob import glob
+from typing import Optional
+
+import fabio
+import numpy
+from lxml import etree
+
+from tomoscan.framereducer.target import REDUCER_TARGET
+from tomoscan.framereducer.framereducerbase import FrameReducerBase
+from tomoscan.framereducer.method import ReduceMethod
+from tomoscan.scanbase import ReducedFramesInfos, TomoScanBase
 
 _logger = logging.getLogger(__name__)
 
@@ -277,12 +277,12 @@ class EDFFrameReducer(FrameReducerBase):
                 fileName = os.path.join(directory, fileName)
                 if os.path.isfile(fileName):
                     if self.overwrite is False:
-                        _logger.info("skip creation of %s, already existing" % fileName)
+                        _logger.info(f"skip creation of {fileName}, already existing")
                         continue
 
                 if self.nFilePerSerie == 1:
                     fSerieName = os.path.join(directory, self.series[i])
-                    header = {"method": self.reduced_method.name + " on 1 image"}
+                    header = {"method": f"{self.reduced_method.name} on 1 image"}
                     header["SRCUR"] = self.get_closest_SR_current(
                         scan_dir=directory, refFile=fSerieName
                     )
@@ -344,8 +344,7 @@ class EDFFrameReducer(FrameReducerBase):
                     return
                 else:
                     raise ValueError(
-                        "Unrecognized calculation type request {}"
-                        "".format(self.reduced_method)
+                        f"Unrecognized calculation type request {self.reduced_method}"
                     )
 
                 if (
@@ -418,8 +417,7 @@ class EDFFrameReducer(FrameReducerBase):
         # if nothing to process
         if self.nfiles == 0:
             _logger.info(
-                "no %s for %s, because no file to compute found"
-                % (self.reducer_target, directory)
+                f"no {self.reducer_target} for {directory}, because no file to compute found"
             )
             return False
 
@@ -508,7 +506,7 @@ class EDFFrameReducer(FrameReducerBase):
             [unattributedFiles.remove(_f) for _f in serieFiles[serie]]
 
         if len(unattributedFiles) > 0:
-            _logger.error("Failed to associate %s to any serie" % unattributedFiles)
+            _logger.error(f"Failed to associate {unattributedFiles} to any serie")
             return {}, 0
 
         checkSeriesFilesLength(serieFiles)
@@ -589,7 +587,7 @@ class EDFFrameReducer(FrameReducerBase):
             key = serievec[-1]
             tomoN = self.getTomo_N(self.scan)
             if tomoN is None:
-                _logger.error("Fail to found information %s. Can't find TOMO_N")
+                _logger.error("Fail to found information TOMO_N")
             del serievec[-1]
             serievec.append(str(tomoN).zfill(4))
             filesPerSerie[serievec[-1]] = filesPerSerie[key]
