@@ -29,17 +29,21 @@ __authors__ = ["H. Payno"]
 __license__ = "MIT"
 __date__ = "30/09/2019"
 
-import h5py
-import numpy
+import logging
 import os
 from xml.etree import cElementTree
+
 import fabio
 import fabio.edfimage
-from .hdf5scan import ImageKey, HDF5TomoScan
-from tomoscan.esrf.volume.hdf5volume import HDF5Volume
+import h5py
+import numpy
 from silx.io.utils import h5py_read_dataset
+from tomoscan.utils.io import deprecated_warning
+
+from tomoscan.esrf.volume.hdf5volume import HDF5Volume
+
+from .nxtomoscan import NXtomoScan, ImageKey
 from .utils import dump_info_file
-import logging
 
 _logger = logging.getLogger(__name__)
 
@@ -156,10 +160,10 @@ class ScanMock:
             background[radii < radius * scale] = 1.0
             return background
         else:
-            raise ValueError("selected scene %s is no managed" % self.scene)
+            raise ValueError(f"selected scene {self.scene} is no managed")
 
 
-class MockHDF5(ScanMock):
+class MockNXtomo(ScanMock):
     """
     Mock an acquisition in a hdf5 file.
 
@@ -247,7 +251,7 @@ class MockHDF5(ScanMock):
         self._z_pos = z_pos
         self._magnification = magnification
 
-        super(MockHDF5, self).__init__(
+        super(MockNXtomo, self).__init__(
             scan_path=scan_path,
             n_radio=n_proj,
             n_ini_radio=n_ini_proj,
@@ -274,7 +278,7 @@ class MockHDF5(ScanMock):
             self.add_distance(distance)
 
         self._define_fov(field_of_view, estimated_cor_frm_motor)
-        self.scan = HDF5TomoScan(scan=self.scan_master_file, entry="entry")
+        self.scan = NXtomoScan(scan=self.scan_master_file, entry="entry")
 
     @property
     def has_intensity_monitor(self):
@@ -720,9 +724,7 @@ class MockEDF(ScanMock):
         else:
             self._last_radio_index += 1
             index_ = self._last_radio_index
-        file_name = (
-            os.path.basename(self.scan_path) + "_{0:04d}".format(index_) + ".edf"
-        )
+        file_name = f"{os.path.basename(self.scan_path)}_{index_:04}.edf"
         f = os.path.join(self.scan_path, file_name)
         if not os.path.exists(f):
             if index_ < len(self._proj_rotation_angles):
@@ -748,7 +750,7 @@ class MockEDF(ScanMock):
             edf_writer.write(f)
 
     def add_dark(self, index):
-        file_name = "darkend{0:04d}.edf".format(index)
+        file_name = f"darkend{index:04}.edf"
         file_path = os.path.join(self.scan_path, file_name)
         if not os.path.exists(file_path):
             data = numpy.random.random((self.det_height * self.det_width)).reshape(
@@ -766,7 +768,7 @@ class MockEDF(ScanMock):
             edf_writer.write(file_path)
 
     def add_flat(self, index):
-        file_name = "refHST{0:04d}.edf".format(index)
+        file_name = f"refHST{index:04}.edf"
         file_path = os.path.join(self.scan_path, file_name)
         if not os.path.exists(file_path):
             data = numpy.random.random((self.det_height * self.det_width)).reshape(
@@ -842,17 +844,17 @@ class MockEDF(ScanMock):
             "\n".join(
                 [
                     "! PyHST_SLAVE VOLUME INFO FILE",
-                    "NUM_X =  %s" % shape[2],
-                    "NUM_Y =  %s" % shape[1],
-                    "NUM_Z =  %s" % shape[0],
-                    "voxelSize =  %s" % voxelSize,
+                    f"NUM_X =  {shape[2]}",
+                    f"NUM_Y =  {shape[1]}",
+                    f"NUM_Z =  {shape[0]}",
+                    f"voxelSize =  {voxelSize}",
                     "BYTEORDER = LOWBYTEFIRST",
-                    "ValMin =  %s" % valMin,
-                    "ValMax =  %s" % valMax,
-                    "s1 =  %s" % s1,
-                    "s2 =  %s" % s2,
-                    "S1 =  %s" % S1,
-                    "S2 =  %s" % S2,
+                    f"ValMin =  {valMin}",
+                    f"ValMax =  {valMax}",
+                    f"s1 =  {s1}",
+                    f"s2 =  {s2}",
+                    f"S1 =  {S1}",
+                    f"S2 =  {S2}",
                 ]
             )
         )
@@ -880,7 +882,7 @@ class MockEDF(ScanMock):
 
         # create scan files
         for i in range((n_radio + n_extra_radio)):
-            file_name = basename + "_{0:04d}".format(i) + ".edf"
+            file_name = f"{basename}_{i:04}.edf"
             f = os.path.join(folder, file_name)
             if not os.path.exists(f):
                 data = numpy.random.random(dim * dim).reshape(dim, dim)
@@ -979,7 +981,7 @@ class MockEDF(ScanMock):
 
     @staticmethod
     def add_flat_serie(scan_path, n_elmt, index, dim, data):
-        ref_file = os.path.join(scan_path, "ref0000_{}.edf".format(str(index).zfill(4)))
+        ref_file = os.path.join(scan_path, f"ref0000_{index:04}.edf")
         if data is None:
             data = numpy.array(
                 numpy.random.random(n_elmt * dim * dim) * 100, numpy.uint32
@@ -992,7 +994,7 @@ class MockEDF(ScanMock):
 
     @staticmethod
     def add_dark_serie(scan_path, n_elmt, index, dim, data):
-        dark_file = os.path.join(scan_path, "darkend{}.edf".format(str(index).zfill(4)))
+        dark_file = os.path.join(scan_path, f"darkend{index:04}.edf")
         if data is None:
             data = numpy.array(
                 numpy.random.random(n_elmt * dim * dim) * 100, numpy.uint32
@@ -1002,3 +1004,44 @@ class MockEDF(ScanMock):
         for frame in data[1:]:
             edf_writer.append_frame(data=frame)
         edf_writer.write(dark_file)
+
+
+class MockHDF5(MockNXtomo):
+    def __init__(
+        self,
+        scan_path,
+        n_radio,
+        n_ini_radio=None,
+        n_extra_radio=0,
+        scan_range=360,
+        n_recons=0,
+        n_pag_recons=0,
+        recons_vol=False,
+        dim=200,
+        ref_n=0,
+        flat_n=0,
+        dark_n=0,
+        scene="noise",
+    ):
+        deprecated_warning(
+            type_="class",
+            name="tomoscan.esrf.scan.mock.MockHDF5",
+            replacement="tomoscan.esrf.scan.mock.MockNXtomo",
+            since_version="2.0",
+            reason="coherence",
+        )
+        super().__init__(
+            scan_path,
+            n_radio,
+            n_ini_radio,
+            n_extra_radio,
+            scan_range,
+            n_recons,
+            n_pag_recons,
+            recons_vol,
+            dim,
+            ref_n,
+            flat_n,
+            dark_n,
+            scene,
+        )
