@@ -1,53 +1,28 @@
-# coding: utf-8
-# /*##########################################################################
-# Copyright (C) 2016-2022 European Synchrotron Radiation Facility
-#
-# Permission is hereby granted, free of charge, to any person obtaining a copy
-# of this software and associated documentation files (the "Software"), to deal
-# in the Software without restriction, including without limitation the rights
-# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-# copies of the Software, and to permit persons to whom the Software is
-# furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included in
-# all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
-# THE SOFTWARE.
-#
-#############################################################################
+"""module dedicated to esrf scans utils"""
 
-
-__authors__ = ["H.Payno"]
-__license__ = "MIT"
-__date__ = "10/10/2019"
-
-
-import os
-import fabio
-from silx.io.url import DataUrl
-from silx.utils.deprecation import deprecated
-from silx.io.utils import h5py_read_dataset, get_data as silx_get_data
-from silx.io.dictdump import h5todict
-from typing import Union
-from typing import Iterable
-import numpy
-import logging
-import sys
-from tomoscan.io import HDF5File
-import warnings
 import contextlib
 import fnmatch
+import logging
+import os
+import sys
+import psutil
+import warnings
+from typing import Iterable, Union, Optional
+import resource
+import traceback
+
+import fabio
 import h5py
+import numpy
+from silx.io.dictdump import h5todict
+from silx.io.url import DataUrl
+from silx.io.utils import get_data as silx_get_data
+from silx.io.utils import h5py_read_dataset
+from silx.utils.deprecation import deprecated
+from silx.io.utils import open as hdf5_open
 
 from tomoscan.scanbase import ReducedFramesInfos, TomoScanBase
 
-
 _logger = logging.getLogger(__name__)
 
 
@@ -77,7 +52,7 @@ def get_parameters_frm_par_or_info(file_: str) -> dict:
         try:
             key, value = line_.split("=")
         except ValueError:
-            raise ValueError('fail to extract information from "%s"' % line_)
+            raise ValueError(f"fail to extract information from {line_}")
         else:
             # try to cast the value on int, else float else don't
             try:
@@ -285,22 +260,137 @@ def grp_is_complete(scans: Iterable) -> bool:
     return serie_is_complete_from_group_size(scans)
 
 
-def dataset_has_broken_vds(url: DataUrl):
+def __get_log_fct(log_level):
+    if log_level is logging.WARNING:
+        return _logger.warning
+    elif log_level is logging.ERROR:
+        return _logger.error
+    elif log_level is logging.DEBUG:
+        return _logger.debug
+    elif log_level is logging.INFO:
+        return _logger.info
+    elif log_level is logging.CRITICAL:
+        return _logger.critical
+    else:
+        raise ValueError("logging level unrecognized")
+
+
+def dataset_has_broken_vds(
+    url: DataUrl,
+    raise_error=False,
+    log_level=logging.WARNING,
+    return_unique_files=False,
+) -> bool:
     """
     check that the provided url is not a VDS with broken links.
+
+
+    :param DataUrl url: url to the dataset to treat
+    :param bool raise_error: if True and dataset not existing will raise an error
+    :param bool return_unique_files: if True return unique files. As this step can be time consuming and reused it can sometimes be convenients
     """
     if not isinstance(url, DataUrl):
         raise TypeError(f"{url} is expected to be an instance of {DataUrl}")
-    with HDF5File(url.file_path(), mode="r") as h5f:
-        dataset = h5f[url.data_path()]
+
+    uniques_files = ()
+
+    with hdf5_open(url.file_path()) as h5f:
+        dataset = h5f.get(url.data_path(), None)
+        if dataset is None:
+            msg = f"no data found at {url.file_path()}://{url.data_path()}"
+            if raise_error:
+                raise ValueError(msg)
+            else:
+                __get_log_fct(log_level)(msg)
+            if return_unique_files:
+                return (True, None)
+            else:
+                return True
+
         if not dataset.is_virtual:
-            return False
+            if return_unique_files:
+                return (False, (url.file_path(),))
+            else:
+                return False
+
         else:
-            for file_ in get_unique_files_linked(url=url):
-                if not os.path.exists(file_):
-                    _logger.warning(f"{file_} does not exists")
+            # free dataset in case it point to another file. Else won't free the object before calling get_unique_files_linked
+            dataset = None
+            h5f.close()
+            uniques_files = get_unique_files_linked(url=url)
+            missing_files = tuple(
+                filter(
+                    lambda file_: not os.path.exists(file_),
+                    uniques_files,
+                )
+            )
+            if len(missing_files) > 0:
+                msg = f"dataset {url.file_path()} has broken virtual-dataset at {url.data_path()}. {missing_files} missing"
+
+                __get_log_fct(log_level)(msg)
+                if raise_error:
+                    raise OSError(msg)
+                if return_unique_files:
+                    return (True, uniques_files)
+                else:
                     return True
-    return False
+
+    if return_unique_files:
+        return (False, uniques_files)
+    else:
+        return False
+
+
+def check_possible_issue_with_rlimit(
+    url: Optional[DataUrl],
+    raise_error=False,
+    log_level=logging.WARNING,
+    delta_n_file=0,
+    substract_current_open=True,
+    unique_files: Optional[tuple] = None,
+) -> bool:
+    """
+    check that the provided url does not contain more external file than (ulimit - delta_ulimit).
+    Else if this limit is reached we will probably met some troubles when reading data.
+    Once this limit is reached - vds data will return 0 only - silently
+    """
+    if unique_files is None and url is None:
+        raise ValueError("'unique_files' or 'url' should be provided")
+
+    if unique_files is None:
+        # first check if dataset is virtual, else skip test
+        with hdf5_open(url.file_path()) as h5f:
+            dataset = h5f.get(url.data_path(), None)
+            if dataset is None:
+                msg = f"no data found at {url.file_path()}://{url.data_path()}"
+                if raise_error:
+                    raise ValueError(msg)
+                else:
+                    __get_log_fct(log_level)(msg)
+                return True
+
+            if not dataset.is_virtual:
+                return False
+
+        unique_files = get_unique_files_linked(url=url)
+
+    n_files = len(unique_files)
+    if substract_current_open:
+        current_process = psutil.Process()
+        n_open_file_currently = len(current_process.open_files())
+    else:
+        n_open_file_currently = 0
+    try:
+        rlimit = resource.getrlimit(resource.RLIMIT_NOFILE)[0]
+    except (ValueError, OSError):
+        _logger.warning("Failed to check_possible_issue_with_rlimit")
+    else:
+        might_met_troubles = n_files > (rlimit - delta_n_file - n_open_file_currently)
+        if might_met_troubles:
+            msg = f"too much external files to open from {url.path()} - contains {n_files} external files. OS rlimit is set to {rlimit}"
+            __get_log_fct(log_level)(msg)
+            if raise_error:
+                raise OSError(msg)
 
 
 def get_datasets_linked_to_vds(url: DataUrl):
@@ -332,12 +422,11 @@ def get_datasets_linked_to_vds(url: DataUrl):
     while len(virtual_dataset_to_treat) > 0:
         to_treat = list(virtual_dataset_to_treat)
         virtual_dataset_to_treat.clear()
-
         for file_path, dataset_path, dataset_slice in to_treat:
             if (file_path, dataset_path, dataset_slice) in already_checked:
                 continue
             if os.path.exists(file_path):
-                with HDF5File(file_path, mode="r") as h5f:
+                with hdf5_open(file_path) as h5f:
                     dataset = h5f[dataset_path]
                     if dataset.is_virtual:
                         for vs_info in dataset.virtual_sources():
@@ -363,18 +452,20 @@ def get_datasets_linked_to_vds(url: DataUrl):
                                 # if those can also contains virtual dataset and we want to handle
                                 # the case a part of it is broken but not ours this should handle
                                 # hyperslab
-                                virtual_dataset_to_treat.add(
-                                    (
-                                        os.path.realpath(vs_info.file_name)
-                                        if vs_info.file_name != "."
-                                        else os.path.abspath(url.file_path()),
-                                        # avoid calling os.path.realpath if the dataset is in the same file. Otherwise mess up with paths
-                                        vs_info.dset_name,
-                                        None,
+                                if vs_info.file_name != ".":
+                                    # do not check self contained dataset
+                                    virtual_dataset_to_treat.add(
+                                        (
+                                            os.path.abspath(vs_info.file_name),
+                                            # avoid calling os.path.abspath if the dataset is in the same file. Otherwise mess up with paths
+                                            str(vs_info.dset_name),
+                                            None,
+                                        )
                                     )
-                                )
                     else:
                         final_dataset.add((file_path, dataset_path, dataset_slice))
+                    dataset = None
+
             else:
                 final_dataset.add((file_path, dataset_path, dataset_slice))
             already_checked.add((file_path, dataset_path, dataset_slice))
@@ -467,9 +558,10 @@ def dump_info_file(
 def cwd_context(new_cwd=None):
     try:
         curdir = os.getcwd()
-    except Exception as e:
-        _logger.error(e)
-        curdir = None
+    except Exception:
+        traceback.print_stack(limit=3)
+        _logger.error("Working directory has been delated. Will move to '~'")
+        curdir = os.path.expanduser("~")
     try:
         if new_cwd is not None and os.path.isfile(new_cwd):
             new_cwd = os.path.dirname(new_cwd)
@@ -485,9 +577,9 @@ def get_data(url: DataUrl):
     # update the current working dircetory for external dataset
     if url.file_path() is not None and h5py.is_hdf5(url.file_path()):
         # convert path to real path to insure it will be constant when changing current working directory
-        file_path = os.path.realpath(url.file_path())
+        file_path = os.path.abspath(url.file_path())
         with cwd_context(file_path):
-            with HDF5File(file_path, mode="r") as h5f:
+            with hdf5_open(file_path) as h5f:
                 if url.data_path() in h5f:
                     if url.data_slice() is None:
                         return h5py_read_dataset(h5f[url.data_path()])
@@ -501,7 +593,11 @@ def get_data(url: DataUrl):
 
 
 def copy_h5_dict_darks_to(
-    scan, darks_url: DataUrl, save=False, raise_error_if_url_empty=True
+    scan,
+    darks_url: DataUrl,
+    save: bool = False,
+    raise_error_if_url_empty: bool = True,
+    overwrite: bool = False,
 ):
     """
     :param TomwerScanBase scan: target to copy darks
@@ -537,11 +633,15 @@ def copy_h5_dict_darks_to(
     data = from_relative_reduced_frames_to_absolute(reduced_frames=data, scan=scan)
     scan.set_reduced_darks(darks=data, darks_infos=metadata)
     if save:
-        scan.save_reduced_darks(darks=data, darks_infos=metadata)
+        scan.save_reduced_darks(darks=data, darks_infos=metadata, overwrite=overwrite)
 
 
 def copy_h5_dict_flats_to(
-    scan, flats_url: DataUrl, save=False, raise_error_if_url_empty=True
+    scan,
+    flats_url: DataUrl,
+    save=False,
+    raise_error_if_url_empty=True,
+    overwrite: bool = False,
 ):
     """
     :param TomwerScanBase scan: target to copy darks
@@ -577,7 +677,7 @@ def copy_h5_dict_flats_to(
     data = from_relative_reduced_frames_to_absolute(reduced_frames=data, scan=scan)
     scan.set_reduced_flats(flats=data, flats_infos=metadata)
     if save:
-        scan.save_reduced_flats(flats=data, flats_infos=metadata)
+        scan.save_reduced_flats(flats=data, flats_infos=metadata, overwrite=overwrite)
 
 
 def from_relative_reduced_frames_to_absolute(reduced_frames: dict, scan: TomoScanBase):
