@@ -30,19 +30,21 @@ __license__ = "MIT"
 __date__ = "27/01/2022"
 
 
-import os
-from typing import Optional
-from tomoscan.scanbase import TomoScanBase
-from tomoscan.volumebase import VolumeBase
-from tomoscan.esrf.identifier.hdf5Identifier import HDF5VolumeIdentifier
-from silx.io.url import DataUrl
-from tomoscan.utils import docstring
-from tomoscan.io import HDF5File
-from silx.io.dictdump import dicttonx, nxtodict
-from silx.utils.deprecation import deprecated_warning
-import numpy
 import logging
+import os
+from typing import Union, Optional
+
 import h5py
+import numpy
+from silx.io.dictdump import dicttonx, nxtodict
+from silx.io.url import DataUrl
+from silx.io.utils import open as open_hdf5
+
+from tomoscan.utils.io import deprecated_warning
+from tomoscan.esrf.identifier.hdf5Identifier import HDF5VolumeIdentifier
+from tomoscan.scanbase import TomoScanBase
+from tomoscan.utils import docstring
+from tomoscan.volumebase import VolumeBase
 
 _logger = logging.getLogger(__name__)
 
@@ -185,7 +187,7 @@ class HDF5Volume(VolumeBase):
 
         if url.file_path() is not None and os.path.dirname(url.file_path()) != "":
             os.makedirs(os.path.dirname(url.file_path()), exist_ok=True)
-        with HDF5File(filename=url.file_path(), mode=mode) as h5s:
+        with h5py.File(url.file_path(), mode=mode) as h5s:
             if url.data_path() in h5s:
                 if self.overwrite:
                     _logger.debug(
@@ -265,7 +267,7 @@ class HDF5Volume(VolumeBase):
             and os.path.dirname(data_url.file_path()) != ""
         ):
             os.makedirs(os.path.dirname(data_url.file_path()), exist_ok=True)
-        with HDF5File(filename=data_url.file_path(), mode=mode) as h5s:
+        with h5py.File(data_url.file_path(), mode=mode) as h5s:
             for i_frame in range(n_frames):
                 yield _FrameDumper(
                     create_dataset=i_frame == 0,
@@ -314,7 +316,7 @@ class HDF5Volume(VolumeBase):
                 "Cannot get data_url. An url should be provided. Don't know where to save this."
             )
 
-        with HDF5File(filename=url.file_path(), mode="r") as h5s:
+        with open_hdf5(filename=url.file_path()) as h5s:
             if url.data_path() in h5s:
                 data = h5s[url.data_path()][()]
             else:
@@ -327,7 +329,7 @@ class HDF5Volume(VolumeBase):
 
     def get_slice(
         self,
-        index=None,
+        index: Union[int, str] = None,
         axis=None,
         xy=None,
         xz=None,
@@ -370,6 +372,16 @@ class HDF5Volume(VolumeBase):
             else:
                 raise ValueError("several axis (previously xy, xz, yz requested")
 
+        if isinstance(index, str):
+            if index == "first":
+                index = 0
+            elif index == "middle":
+                index = self.get_volume_shape()[axis] // 2
+            elif index == "last":
+                index = -1
+            else:
+                raise ValueError(f"index '{index}' is not handled")
+
         if self.data is not None:
             return self.select(volume=self.data, axis=axis, index=index)
         else:
@@ -378,7 +390,7 @@ class HDF5Volume(VolumeBase):
                 raise ValueError(
                     "Cannot get data_url. An url should be provided. Don't know where to save this."
                 )
-            with HDF5File(filename=url.file_path(), mode="r") as h5s:
+            with open_hdf5(filename=url.file_path()) as h5s:
                 if url.data_path() in h5s:
                     return self.select(
                         volume=h5s[url.data_path()], axis=axis, index=index
@@ -461,18 +473,18 @@ class HDF5Volume(VolumeBase):
     @docstring(VolumeBase)
     def browse_slices(self, url=None):
         if url is None and self.data is not None:
-            for slice in self.data:
-                yield slice
+            for data_slice in self.data:
+                yield data_slice
         else:
             url = url or self.data_url
             if url is None:
                 raise ValueError(
                     "No data and data_url know and no url provided. Uanble to browse slices"
                 )
-            with HDF5File(filename=url.file_path(), mode="r") as h5s:
+            with open_hdf5(filename=url.file_path()) as h5s:
                 if url.data_path() in h5s:
-                    for slice in h5s[url.data_path()]:
-                        yield slice
+                    for data_slice in h5s[url.data_path()]:
+                        yield data_slice
                 else:
                     raise KeyError(f"Data path {url.data_path()} not found.")
 
@@ -481,20 +493,20 @@ class HDF5Volume(VolumeBase):
         url = url or self.data_url
         if url is None:
             raise ValueError("Cannot get data_url. An url should be provided.")
-        with HDF5File(filename=url.file_path(), mode="r") as h5s:
+        with open_hdf5(filename=url.file_path()) as h5s:
             if url.data_path() in h5s:
                 return h5s[url.data_path()][chunk]
             else:
                 raise KeyError(f"Data path {url.data_path()} not found.")
 
     def get_volume_shape(self, url=None):
+        if self.data is not None:
+            return self.data.shape
         url = url or self.data_url
         if url is None:
             raise ValueError("Cannot get data_url. An url should be provided.")
-        if self.data is not None:
-            return self.data.shape
         else:
-            with HDF5File(filename=url.file_path(), mode="r") as h5s:
+            with open_hdf5(filename=url.file_path()) as h5s:
                 if url.data_path() in h5s:
                     return h5s[url.data_path()].shape
                 else:
