@@ -30,20 +30,22 @@ __license__ = "MIT"
 __date__ = "10/01/2023"
 
 
-import sys
-import os
-from xml.dom.minidom import parseString as parse_xml_string
 import logging
+import os
+import sys
 from typing import Optional
-import numpy
+from xml.dom.minidom import parseString as parse_xml_string
+
 import h5py
+import numpy
 from dicttoxml import dicttoxml
-from silx.io.url import DataUrl
 from silx.io.dictdump import dicttoini
-from tomoscan.scanbase import TomoScanBase
-from tomoscan.volumebase import VolumeBase
+from silx.io.url import DataUrl
+
 from tomoscan.esrf.identifier.rawidentifier import RawVolumeIdentifier
+from tomoscan.scanbase import TomoScanBase
 from tomoscan.utils import docstring
+from tomoscan.volumebase import VolumeBase
 
 _logger = logging.getLogger(__name__)
 
@@ -194,7 +196,9 @@ class RawVolume(VolumeBase):
             data.tofile(fdesc)
 
     @docstring(VolumeBase)
-    def load_data(self, url: Optional[DataUrl] = None, store: bool = True) -> None:
+    def load_data(
+        self, url: Optional[DataUrl] = None, store: bool = True
+    ) -> numpy.array:
         url = url or self.data_url
         if url is None:
             raise ValueError(
@@ -349,6 +353,29 @@ class RawVolume(VolumeBase):
             self.metadata = metadata
         return metadata
 
+    def get_volume_shape(self, url=None):
+        metadata = self.metadata or self.load_metadata()
+        from_metadata = (
+            metadata.get("SIZEZ"),
+            metadata.get("SIZEY"),
+            metadata.get("SIZEX"),
+        )
+        if (
+            from_metadata[0] is not None
+            and from_metadata[1] is not None
+            and from_metadata[2] is not None
+        ):
+            return from_metadata
+        else:
+            if self.data is None:
+                data = self.load_data(url=url)
+            else:
+                data = self.data
+            if data is not None:
+                return data.shape
+            else:
+                return None
+
     def browse_metadata_files(self, url=None):
         """
         return a generator go through all the existings files associated to the data volume
@@ -369,6 +396,15 @@ class RawVolume(VolumeBase):
         elif url.file_path() is not None and os.path.exists(url.file_path()):
             yield url.file_path()
 
+    def browse_slices(self, url=None):
+        if url is not None or self.data is None:
+            data = self.load_data(url=url)
+        else:
+            data = self.data
+
+        for vol_slice in data:
+            yield vol_slice
+
     def browse_data_urls(self, url=None):
         url = url or self.data_url
         if url is not None and os.path.exists(url.file_path()):
