@@ -30,18 +30,21 @@ __license__ = "MIT"
 __date__ = "01/02/2022"
 
 
+import os
 from typing import Optional
+
 import numpy
+from silx.io.dictdump import dicttoini
+from silx.io.dictdump import load as load_ini
+from silx.io.url import DataUrl
+
 from tomoscan.esrf.identifier.tiffidentifier import (
     MultiTiffVolumeIdentifier,
     TIFFVolumeIdentifier,
 )
-from tomoscan.scanbase import TomoScanBase
 from tomoscan.esrf.volume.singleframebase import VolumeSingleFrameBase
+from tomoscan.scanbase import TomoScanBase
 from tomoscan.utils import docstring, get_subvolume_shape
-from silx.io.url import DataUrl
-from silx.io.dictdump import dicttoini, load as load_ini
-import os
 from tomoscan.volumebase import VolumeBase
 
 try:
@@ -252,8 +255,8 @@ class MultiTIFFVolume(VolumeBase):
                 if self.data.ndim == 2:
                     tif.write(self.data)
                 elif self.data.ndim == 3:
-                    for slice in self.data:
-                        tif.write(slice)
+                    for data_slice in self.data:
+                        tif.write(data_slice)
                 else:
                     raise ValueError(f"data should be 3D and not {self.data.ndim}D")
 
@@ -412,8 +415,8 @@ class MultiTIFFVolume(VolumeBase):
     @docstring(VolumeBase)
     def browse_slices(self, url=None):
         if url is None and self.data is not None:
-            for slice in self.data:
-                yield slice
+            for data_slice in self.data:
+                yield data_slice
         else:
             url = url or self.data_url
             if url is None:
@@ -429,8 +432,8 @@ class MultiTIFFVolume(VolumeBase):
                 for serie in reader.series:
                     data = serie.asarray()
                     if data.ndim == 3:
-                        for slice in data:
-                            yield slice
+                        for data_slice in data:
+                            yield data_slice
                     elif data.ndim == 2:
                         yield data
                     else:
