@@ -28,198 +28,45 @@ __license__ = "MIT"
 __date__ = "09/10/2019"
 
 
-import fabio
-import os
-import typing
 import logging
+import os
+import pathlib
+import typing
+from bisect import bisect_left
+from collections import OrderedDict
+from math import ceil
+from typing import Iterable, Optional, Union
+
+import fabio
 import h5py
 import numpy
-from typing import Union, Iterable, Optional
-from collections import OrderedDict
-import pathlib
-from tomoscan.identifier import ScanIdentifier
-from tomoscan.io import HDF5File
-from tomoscan.unitsystem.electriccurrentsystem import ElectricCurrentSystem
-from tomoscan.unitsystem.timesystem import TimeSystem
-from .unitsystem.metricsystem import MetricSystem
-from silx.utils.enum import Enum as _Enum
+import silx.io.utils
 from silx.io.url import DataUrl
 from silx.io.utils import get_data
-import silx.io.utils
-from math import ceil
+from silx.io.utils import open as hdf5_open
+from tomoscan.utils.io import deprecated, deprecated_warning
+
+from nxtomo.nxobject.nxdetector import FOV
+from tomoscan.identifier import ScanIdentifier
+from tomoscan.normalization import IntensityNormalization
+from tomoscan.normalization import Method as _IntensityMethod
+from tomoscan.normalization import normalize_chebyshev_2D, normalize_lsqr_spline_2D
+from tomoscan.framereducer.reducedframesinfos import ReducedFramesInfos
+from pyunitsystem.electriccurrentsystem import ElectricCurrentSystem
+from pyunitsystem.timesystem import TimeSystem
+from pyunitsystem.metricsystem import MetricSystem
+
 from .progress import Progress
-from bisect import bisect_left
-from tomoscan.normalization import (
-    IntensityNormalization,
-    Method as _IntensityMethod,
-    normalize_chebyshev_2D,
-    normalize_lsqr_spline_2D,
-)
-from silx.utils.deprecation import deprecated
 from .tomoobject import TomoObject
+from .source import *  # noqa F401
 
 _logger = logging.getLogger(__name__)
 
 
-class FOV(_Enum):
-    """Possible existing field of view"""
-
-    @classmethod
-    def from_value(cls, value):
-        if isinstance(value, str):
-            value = value.lower().title()
-
-        return super().from_value(value)
-
-    FULL = "Full"
-    HALF = "Half"
-
-
 # keep compatibility for some time
 _FOV = FOV
 
 
-class SourceType(_Enum):
-    SPALLATION_NEUTRON = "Spallation Neutron Source"
-    PULSED_REACTOR_NEUTRON_SOURCE = "Pulsed Reactor Neutron Source"
-    REACTOR_NEUTRON_SOURCE = "Reactor Neutron Source"
-    SYNCHROTRON_X_RAY_SOURCE = "Synchrotron X-ray Source"
-    PULSED_MUON_SOURCE = "Pulsed Muon Source"
-    ROTATING_ANODE_X_RAY = "Rotating Anode X-ray"
-    FIXED_TUBE_X_RAY = "Fixed Tube X-ray"
-    UV_LASER = "UV Laser"
-    FREE_ELECTRON_LASER = "Free-Electron Laser"
-    OPTICAL_LASER = "Optical Laser"
-    ION_SOURCE = "Ion Source"
-    UV_PLASMA_SOURCE = "UV Plasma Source"
-    METAL_JET_X_RAY = "Metal Jet X-ray"
-
-
-class Source:
-    """Information regarding the x-ray storage ring/facility"""
-
-    def __init__(self, name=None, type=None):
-        self._name = name
-        self._type = type
-
-    @property
-    def name(self) -> Union[None, str]:
-        return self._name
-
-    @name.setter
-    def name(self, name: Union[str, None]):
-        if not isinstance(name, (str, type(None))):
-            raise TypeError("name is expected to be None or a str")
-        self._name = name
-
-    @property
-    def type(self) -> Union[None, SourceType]:
-        return self._type
-
-    @type.setter
-    def type(self, type_: Union[None, str, SourceType]):
-        if type_ is None:
-            self._type = None
-        else:
-            type_ = SourceType.from_value(type_)
-            self._type = type_
-
-    def __str__(self):
-        return f"source (name: {self.name}, type: {self.type})"
-
-
-class ComputeMethod(_Enum):
-    MEAN = "mean"  # compute the mean of dark / flat frames serie
-    MEDIAN = "median"  # compute the median of dark / flat frames serie
-    FIRST = "first"  # take the first frame of the dark / flat serie
-    LAST = "last"  # take the last frame of the dark / flat serie
-
-
-class ReducedFramesInfos:
-    """contains reduced frames metadata as count_time and machine_electric_current"""
-
-    MACHINE_ELECT_CURRENT_KEY = "machine_electric_current"
-
-    COUNT_TIME_KEY = "count_time"
-
-    def __init__(self) -> None:
-        self._count_time = []
-        self._machine_electric_current = []
-
-    def __eq__(self, __o: object) -> bool:
-        if isinstance(__o, dict):
-            return ReducedFramesInfos().load_from_dict(__o) == self
-        if not isinstance(__o, ReducedFramesInfos):
-            return False
-        return numpy.array_equal(
-            numpy.array(self.count_time), numpy.array(__o.count_time)
-        ) and numpy.array_equal(
-            numpy.array(self.machine_electric_current),
-            numpy.array(__o.machine_electric_current),
-        )
-
-    def clear(self):
-        self._count_time.clear()
-        self._machine_electric_current.clear()
-
-    @property
-    def count_time(self) -> list:
-        """
-        frame exposure time in second
-        """
-        return self._count_time
-
-    @count_time.setter
-    def count_time(self, count_time: Optional[Iterable]):
-        if count_time is None:
-            self._count_time.clear()
-        else:
-            self._count_time = list(count_time)
-
-    @property
-    def machine_electric_current(self) -> list:
-        """
-        machine electric current in Ampere
-        """
-        return self._machine_electric_current
-
-    @machine_electric_current.setter
-    def machine_electric_current(self, machine_electric_current: Optional[Iterable]):
-        if machine_electric_current is None:
-            self._machine_electric_current.clear()
-        else:
-            self._machine_electric_current = list(machine_electric_current)
-
-    def to_dict(self) -> dict:
-        res = {}
-        if len(self.machine_electric_current) > 0:
-            res[self.MACHINE_ELECT_CURRENT_KEY] = self.machine_electric_current
-        if len(self.count_time) > 0:
-            res[self.COUNT_TIME_KEY] = self.count_time
-        return res
-
-    def load_from_dict(self, my_dict: dict):
-        self.machine_electric_current = my_dict.get(
-            self.MACHINE_ELECT_CURRENT_KEY, None
-        )
-        self.count_time = my_dict.get(self.COUNT_TIME_KEY, None)
-        return self
-
-    @staticmethod
-    def pop_info_keys(my_dict: dict):
-        if not isinstance(my_dict, dict):
-            raise TypeError
-        my_dict.pop(ReducedFramesInfos.MACHINE_ELECT_CURRENT_KEY, None)
-        my_dict.pop(ReducedFramesInfos.COUNT_TIME_KEY, None)
-        return my_dict
-
-    @staticmethod
-    def split_data_and_metadata(my_dict):
-        metadata = ReducedFramesInfos().load_from_dict(my_dict)
-        data = ReducedFramesInfos.pop_info_keys(my_dict)
-        return data, metadata
-
-
 class TomoScanBase(TomoObject):
     """
     Base Class representing a scan.
@@ -277,11 +124,19 @@ class TomoScanBase(TomoObject):
         """Extra information for normalization"""
         self._electric_current = None
         self._count_time = None
+        self._source_type = None
+        self._source_name = None
+        self._instrument_name = None
+        self._title = None
 
     def clear_caches(self):
         """clear caches. Might be call if some data changed after
         first read of data or metadata"""
         self._notify_ffc_rsc_missing = True
+        self._source_type = None
+        self._source_name = None
+        self._instrument_name = None
+        self._title = None
         self.clear_frames_caches()
 
     def clear_frames_caches(self):
@@ -375,7 +230,7 @@ class TomoScanBase(TomoObject):
                 raise TypeError(
                     f"path is expected to be a str or a pathlib.Path not {type(path)}"
                 )
-            self._path = os.path.realpath(str(path))
+            self._path = os.path.abspath(str(path))
 
     @property
     def type(self) -> str:
@@ -460,6 +315,7 @@ class TomoScanBase(TomoObject):
 
     @property
     def flat_n(self) -> Union[None, int]:
+        """number of flat per serie (computed on the first serie)"""
         raise NotImplementedError("Base class")
 
     @property
@@ -615,9 +471,22 @@ class TomoScanBase(TomoObject):
             )
         self._electric_current = current
 
+    @property
+    def title(self):
+        raise NotImplementedError("Base class")
+
+    @property
+    def source_name(self):
+        raise NotImplementedError("Base class")
+
+    @property
+    def source_type(self):
+        raise NotImplementedError("Base class")
+
     @property
     def x_flipped(self) -> bool:
         """
+        warning: deprecated !!!!!
         return True if  the frames are flip through x
         """
         raise NotImplementedError("Base class")
@@ -625,22 +494,48 @@ class TomoScanBase(TomoObject):
     @property
     def y_flipped(self) -> bool:
         """
+        warning: deprecated !!!!!
         return True if  the frames are flip through y
         """
         raise NotImplementedError("Base class")
 
     def get_x_flipped(self, default=None):
+        deprecated_warning(
+            type_="property",
+            name="get_x_flipped",
+            replacement="get_detector_transformations",
+            since_version="1.3",
+        )
         if self.x_flipped is None:
             return default
         else:
             return self.x_flipped
 
     def get_y_flipped(self, default=None):
+        deprecated_warning(
+            type_="property",
+            name="get_y_flipped",
+            replacement="get_detector_transformations",
+            since_version="1.3",
+        )
         if self.y_flipped is None:
             return default
         else:
             return self.y_flipped
 
+    @property
+    def detector_transformations(self) -> Optional[tuple]:
+        """
+        return tuple of `Transformation` applied to the detector
+        """
+        raise NotImplementedError
+
+    def get_detector_transformations(self, default):
+        if self.detector_transformations is None:
+            return default
+        else:
+            return self.detector_transformations
+
     def get_identifier(self) -> ScanIdentifier:
         """
         return the dataset identifier of the scan.
@@ -826,18 +721,15 @@ class TomoScanBase(TomoObject):
             and self.dim_2 is not None
             and line > self.dim_2
         ) or line < 0:
-            raise ValueError("requested line {} is not in the scan".format(line))
+            raise ValueError(f"requested line {line} is not in the scan")
         if self.projections is not None:
             y_dim = ceil(len(self.projections) / subsampling)
             sinogram = numpy.empty((y_dim, self.dim_1))
             _logger.debug(
-                "compute sinogram for line {} of {} (subsampling: {})".format(
-                    line, self.path, subsampling
-                )
+                f"compute sinogram for line {line} of {self.path} (subsampling: {subsampling})"
             )
             advancement = Progress(
-                name="compute sinogram for {}, line={},"
-                "sampling={}".format(os.path.basename(self.path), line, subsampling)
+                name=f"compute sinogram for {os.path.basename(self.path)}, line={line},sampling={subsampling}"
             )
             advancement.setMaxAdvancement(len(self.projections))
             projections = self.projections
@@ -1091,9 +983,11 @@ class TomoScanBase(TomoObject):
                 self._frame_flat_field_correction(
                     data=frame,
                     dark=dark,
-                    flat_weights=self._flats_weights[proj_i]
-                    if proj_i in self._flats_weights
-                    else None,
+                    flat_weights=(
+                        self._flats_weights[proj_i]
+                        if proj_i in self._flats_weights
+                        else None
+                    ),
                     line=line,
                 )
                 for frame, proj_i in zip(projs, proj_indexes)
@@ -1189,6 +1083,7 @@ class TomoScanBase(TomoObject):
         output_urls,
         frames_metadata: Optional[ReducedFramesInfos],
         metadata_output_urls: Optional[tuple],
+        overwrite: bool = False,
     ):
         """
         utils function to save some frames in set of output_urls
@@ -1251,8 +1146,10 @@ class TomoScanBase(TomoObject):
                         url, entry=entry, idx=idx, idx_zfill4=idx_zfill4
                     ).split("/")[-1]
                 )
-            with HDF5File(file_path, mode="a") as h5s:
+            with h5py.File(file_path, mode="a") as h5s:
                 if group_path in h5s:
+                    if not overwrite:
+                        raise KeyError("group_path already exists")
                     for key in h5s[group_path].keys():
                         if key not in used_datasets:
                             del h5s[group_path][key]
@@ -1321,7 +1218,8 @@ class TomoScanBase(TomoObject):
                         )
                         edf_writer.write(file_path)
                 elif scheme in ("hdf5", "silx"):
-                    with HDF5File(file_path, mode="a") as h5s:
+                    os.makedirs(os.path.dirname(file_path), exist_ok=True)
+                    with h5py.File(file_path, mode="a") as h5s:
                         if data_path in h5s:
                             h5s[data_path][()] = frame
                         else:
@@ -1350,7 +1248,7 @@ class TomoScanBase(TomoObject):
                     else:
                         # save metadata
                         if scheme in ("hdf5", "silx"):
-                            with HDF5File(file_path, mode="a") as h5s:
+                            with h5py.File(file_path, mode="a") as h5s:
                                 metadata_path = "/".join(
                                     [metadata_grp_path, metadata_name]
                                 )
@@ -1495,7 +1393,7 @@ class TomoScanBase(TomoObject):
 
                     if not os.path.exists(frame_file_path):
                         continue
-                    with HDF5File(frame_file_path, mode="r") as h5s:
+                    with hdf5_open(frame_file_path) as h5s:
                         dataset_or_group = h5s[data_path]
                         if isinstance(dataset_or_group, h5py.Dataset):
                             idx = None
@@ -1568,7 +1466,7 @@ class TomoScanBase(TomoObject):
                 if scheme in ("hdf5", "silx"):
                     if not os.path.exists(metadata_file):
                         continue
-                    with HDF5File(metadata_file, mode="r") as h5s:
+                    with hdf5_open(metadata_file) as h5s:
                         if data_path not in h5s:
                             continue
                         parent_group = h5s[data_path]
@@ -1622,6 +1520,7 @@ class TomoScanBase(TomoObject):
         output_urls: tuple,
         darks_infos: Optional[ReducedFramesInfos] = None,
         metadata_output_urls: Optional[tuple] = None,
+        overwrite: bool = False,
     ) -> None:
         """
         Dump computed dark (median / mean...) into files
@@ -1631,6 +1530,7 @@ class TomoScanBase(TomoObject):
             output_urls=output_urls,
             frames_metadata=darks_infos,
             metadata_output_urls=metadata_output_urls,
+            overwrite=overwrite,
         )
 
     def load_reduced_darks(
@@ -1659,6 +1559,7 @@ class TomoScanBase(TomoObject):
         output_urls: tuple,
         flats_infos: Optional[ReducedFramesInfos] = None,
         metadata_output_urls: Optional[tuple] = None,
+        overwrite: bool = False,
     ) -> None:
         """
         Dump reduced flats (median / mean...) into files
@@ -1668,6 +1569,7 @@ class TomoScanBase(TomoObject):
             output_urls=output_urls,
             frames_metadata=flats_infos,
             metadata_output_urls=metadata_output_urls,
+            overwrite=overwrite,
         )
 
     def load_reduced_flats(
