@@ -28,15 +28,15 @@ __authors__ = ["H. Payno"]
 __license__ = "MIT"
 __date__ = "24/08/2021"
 
-import numpy
-
-from tomoscan.scanbase import TomoScanBase
-from tomoscan.esrf.scan.utils import dataset_has_broken_vds
-from tomoscan.esrf.scan.utils import get_compacted_dataslices
-from silx.io.utils import get_data
 import logging
-from typing import Optional
 import weakref
+from typing import Optional
+
+import numpy
+from silx.io.utils import get_data
+
+from tomoscan.esrf.scan.utils import dataset_has_broken_vds, get_compacted_dataslices
+from tomoscan.scanbase import TomoScanBase
 
 _logger = logging.getLogger(__name__)
 
@@ -367,9 +367,9 @@ class PixelValidator(_ScanParamValidator):
         )
 
     def _run(self) -> Optional[bool]:
-        from tomoscan.esrf.hdf5scan import HDF5TomoScan
+        from tomoscan.esrf.scan.nxtomoscan import NXtomoScan
 
-        if isinstance(self.scan, HDF5TomoScan):
+        if isinstance(self.scan, NXtomoScan):
             return (self.scan.x_pixel_size not in (None, 0)) and (
                 self.scan.y_pixel_size not in (None, 0)
             )
