@@ -30,18 +30,22 @@ __license__ = "MIT"
 __date__ = "27/01/2022"
 
 
-import numpy
-from tomoscan.identifier import VolumeIdentifier
-from tomoscan.tomoobject import TomoObject
-from tomoscan.scanbase import TomoScanBase
-from tomoscan.utils import BoundingBox1D, BoundingBox3D
-from tomoscan.unitsystem.metricsystem import MetricSystem
 from typing import Optional, Union
+import logging
+import numpy
 from silx.io.url import DataUrl
 from silx.math.combo import (  # pylint: disable=E0611 (not found from the pyx file)
     min_max,
 )
-from silx.utils.deprecation import deprecated_warning
+from pyunitsystem.metricsystem import MetricSystem
+
+from tomoscan.utils.io import deprecated, deprecated_warning
+from tomoscan.identifier import VolumeIdentifier
+from tomoscan.scanbase import TomoScanBase
+from tomoscan.tomoobject import TomoObject
+from tomoscan.utils import BoundingBox1D, BoundingBox3D
+
+_logger = logging.getLogger(__name__)
 
 
 class VolumeBase(TomoObject):
@@ -156,7 +160,7 @@ class VolumeBase(TomoObject):
 
     def get_slice(
         self,
-        index=None,
+        index: Union[str, int] = None,
         axis=None,
         xy=None,
         xz=None,
@@ -173,6 +177,14 @@ class VolumeBase(TomoObject):
             # in the worst case.
             self.load_data(url=url, store=True)
 
+        if isinstance(index, str):
+            if index == "middle":
+                index = self.get_volume_shape()[axis] // 2
+            elif index == "first":
+                index = 0
+            elif index == "last":
+                index = -1
+
         if self.data is not None:
             return self.select(
                 volume=self.data, xy=xy, xz=xz, yz=yz, axis=axis, index=index
@@ -238,14 +250,20 @@ class VolumeBase(TomoObject):
             ddict["processing_options"]["reconstruction"] = {}
 
     @property
-    def position(self) -> tuple:
-        """position are provided as a tuple using the same reference for axis as the volume data"""
+    def position(self) -> Optional[tuple]:
+        """position are provided as a tuple using the same reference for axis as the volume data.
+        position is returned as (axis_0_pos, axis_1_pos, axis_2_pos). Can also be see as (z_position, y_position, x_position)
+        """
         metadata = self.metadata or self.load_metadata()
-        return tuple(
+        position = (
             metadata.get("processing_options", {})
             .get("reconstruction", {})
             .get("position", None)
         )
+        if position is None:
+            return None
+        else:
+            return tuple(position)
 
     @position.setter
     def position(self, position) -> None:
@@ -257,35 +275,69 @@ class VolumeBase(TomoObject):
         )
 
     @property
+    @deprecated(replacement="voxel_size", since_version="1.3.0")
     def pixel_size(self):
-        metadata = self.metadata or self.load_metadata()
-        pixel_size = (
-            metadata.get("processing_options", {})
-            .get("reconstruction", {})
-            .get("pixel_size_cm", None)
-        )
-        if pixel_size is not None:
-            return pixel_size * MetricSystem.CENTIMETER.value
-        else:
-            return None
+        return self.voxel_size
 
     @pixel_size.setter
+    @deprecated(replacement="voxel_size", since_version="1.3.0")
     def pixel_size(self, pixel_size) -> None:
+        if numpy.isscalar(pixel_size):
+            pixel_size = [pixel_size] * 3
+            _logger.warning(
+                "pixel_size is expected to be a tuple. Conversion will be removed soon. Please update your code"
+            )
+        self.voxel_size = pixel_size
+
+    @property
+    def voxel_size(self) -> Optional[tuple]:
+        """
+        voxel size as (axis 0 dim - aka z, axis 1 dim - aka y, axis 2 dim aka z)
+        """
+        metadata = self.metadata or self.load_metadata()
+
+        voxel_size = (
+            metadata.get("processing_options", {})
+            .get("reconstruction", {})
+            .get("voxel_size_cm", None)
+        )
+        if voxel_size is None:
+            # ensure backward compatibility with old volumes (before nabu 2023.1)
+            # try fall back on pixel_size_cm for old volumes
+            voxel_size = (
+                metadata.get("processing_options", {})
+                .get("reconstruction", {})
+                .get("pixel_size_cm", None)
+            )
+            if voxel_size is not None:
+                voxel_size = (voxel_size, voxel_size, voxel_size)
+
+        if voxel_size is not None:
+            if numpy.isscalar(voxel_size):
+                # be safer and handle the case voxel size is a scalar
+                voxel_size = [voxel_size] * 3
+            return tuple(numpy.array(voxel_size) * MetricSystem.CENTIMETER.value)
+        return None
+
+    @voxel_size.setter
+    def voxel_size(self, voxel_size: tuple) -> None:
         if self.metadata is None:
             self.metadata = {}
+        if numpy.isscalar(voxel_size):
+            raise TypeError("voxel is expected to be a tuple of three values")
         self._insure_reconstruction_dict_exists(self.metadata)
-        self.metadata["processing_options"]["reconstruction"]["pixel_size_cm"] = (
-            pixel_size / MetricSystem.CENTIMETER.value
+        self.metadata["processing_options"]["reconstruction"]["voxel_size_cm"] = (
+            numpy.array(voxel_size) / MetricSystem.CENTIMETER.value
         )
 
-    def get_volume_shape(self):
+    def get_volume_shape(self, url=None):
         raise NotImplementedError("Base class")
 
     def get_bounding_box(self, axis: Optional[Union[str, int]] = None):
         if axis is None:
-            x_bb = self.get_bounding_box(axis="x")
-            y_bb = self.get_bounding_box(axis="y")
-            z_bb = self.get_bounding_box(axis="z")
+            x_bb = self.get_bounding_box(axis=2)
+            y_bb = self.get_bounding_box(axis=1)
+            z_bb = self.get_bounding_box(axis=0)
             return BoundingBox3D(
                 (z_bb.min, y_bb.min, x_bb.min),
                 (z_bb.max, y_bb.max, x_bb.max),
@@ -294,33 +346,33 @@ class VolumeBase(TomoObject):
         position = self.position
         shape = self.get_volume_shape()
         # TODO: does it make sense that pixel size is a scalar ?
-        pixel_size = self.pixel_size
+        voxel_size = self.voxel_size
         missing = []
         if position is None:
             missing.append("position")
         if shape is None:
             missing.append("shape")
             raise ValueError("Unable to find volume shape")
-        if pixel_size is None:
+        if voxel_size is None:
             missing.append("pixel_size")
 
         if len(missing) > 0:
             raise ValueError(
                 f"Unable to get bounding box. Missing information: {'; '.join(missing)}"
             )
-        else:
-            assert axis is not None
+        assert axis is not None
+        if isinstance(axis, str):
             if axis == "x":
-                idx = 2
+                axis = 2
             elif axis == "y":
-                idx = 1
+                axis = 1
             elif axis == "z":
-                idx = 0
+                axis = 0
             else:
                 raise ValueError(f"axis '{axis}' is not handled")
-            min_pos_in_meter = position[idx] - pixel_size * shape[idx] / 2.0
-            max_pos_in_meter = position[idx] + pixel_size * shape[idx] / 2.0
-            return BoundingBox1D(min_pos_in_meter, max_pos_in_meter)
+        min_pos_in_meter = position[axis] - voxel_size[axis] * shape[axis] / 2.0
+        max_pos_in_meter = position[axis] + voxel_size[axis] * shape[axis] / 2.0
+        return BoundingBox1D(min_pos_in_meter, max_pos_in_meter)
 
     def get_min_max(self) -> tuple:
         """
@@ -487,12 +539,12 @@ class VolumeBase(TomoObject):
             data = self.data
         else:
             data = self.browse_slices(url=url)
-        for slice in data:
+        for data_slice in data:
             if min_v is None:
-                min_v = slice.min()
-                max_v = slice.max()
+                min_v = data_slice.min()
+                max_v = data_slice.max()
             else:
-                min_lv, max_lv = min_max(slice, finite=True)
+                min_lv, max_lv = min_max(data_slice, finite=True)
                 min_v = min(min_v, min_lv)
                 max_v = max(max_v, max_lv)
         return min_v, max_v
